const uuid = require('uuid')
const { get } = require('lodash')
const crypto = require('crypto')

class User {
  constructor({ UserModel = {} }) {
    this.UserModel = UserModel
  }

  async createUser({ role, body }) {
    const { UserModel } = this
    const { email, firstName, lastName, affiliation, title } = body
    const username = email
    const password = uuid.v4()
    const userBody = {
      username,
      email,
      password,
      passwordResetToken: crypto.randomBytes(32).toString('hex'),
      isConfirmed: false,
      firstName,
      lastName,
      affiliation,
      title,
      editorInChief: role === 'editorInChief',
      admin: role === 'admin',
      handlingEditor: role === 'handlingEditor',
      invitationToken: role === 'reviewer' ? uuid.v4() : '',
      isActive: true,
      notifications: {
        email: {
          system: true,
          user: true,
        },
      },
    }

    let newUser = new UserModel(userBody)

    newUser = await newUser.save()
    return newUser
  }

  async getEditorsInChief() {
    const { UserModel } = this
    const users = await UserModel.all()

    const eics = users.filter(user => user.editorInChief)
    if (eics.length === 0) {
      throw new Error('No Editor in Chief has been found')
    }

    return eics
  }

  async updateUserTeams({ userId, teamId }) {
    const user = await this.UserModel.find(userId)
    user.teams.push(teamId)
    user.save()
  }

  async getActiveAuthors(authors) {
    const activeUsers = (await Promise.all(
      authors.map(author => this.UserModel.find(author.id)),
    ))
      .filter(user => user.isActive)
      .filter(user => get(user, 'notifications.email.user'))
      .map(user => user.id)

    return authors.filter(author => activeUsers.includes(author.id))
  }

  async getEiCName() {
    const editorsInChief = await this.getEditorsInChief()
    const firstName = get(editorsInChief, '0.firstName', 'Editor')
    const lastName = get(editorsInChief, '0.lastName', 'in Chief')
    return `${firstName} ${lastName}`
  }
}

module.exports = User
