const config = require('config')

const resetPath = config.get('invite-reset-password.url')
const unsubscribeSlug = config.get('unsubscribe.url')

const {
  User,
  Email,
  services,
  Fragment,
} = require('pubsweet-component-helper-service')

const { getEmailCopy } = require('./emailCopy')

module.exports = {
  async sendNotifications({
    user,
    baseUrl,
    fragment,
    reqUserId,
    UserModel,
    collection,
  }) {
    const fragmentHelper = new Fragment({ fragment })
    const { title } = await fragmentHelper.getFragmentData({
      handlingEditor: collection.handlingEditor,
    })
    const { submittingAuthor } = await fragmentHelper.getAuthorData({
      UserModel,
    })

    const titleText = `the manuscript titled "${title}" by ${
      submittingAuthor.firstName
    } ${submittingAuthor.lastName}`

    const userHelper = new User({ UserModel })
    const subjectBaseText = `${collection.customId}: Manuscript`

    const email = new Email({
      type: 'user',
      content: {
        ctaLink: baseUrl,
        ctaText: 'VIEW DASHBOARD',
        signatureName: await userHelper.getEiCName(),
      },
    })

    if (!user.isConfirmed) {
      sendNewAuthorEmail({
        email,
        user,
        baseUrl,
        titleText,
        subjectBaseText,
      })
    }

    const requestUser = await UserModel.find(reqUserId)
    if (requestUser.id !== user.id) {
      sendAddedToManuscriptEmail({
        email,
        baseUrl,
        user,
        titleText,
        subjectBaseText,
      })
    }
  },
}

const sendAddedToManuscriptEmail = ({
  email,
  baseUrl,
  user,
  titleText,
  subjectBaseText,
}) => {
  email.toUser = {
    email: user.email,
    name: `${user.firstName} ${user.lastName}`,
  }

  email.content.subject = `${subjectBaseText} Created`
  email.content.unsubscribeLink = services.createUrl(baseUrl, unsubscribeSlug, {
    id: user.id,
  })

  const { html, text } = email.getBody({
    body: getEmailCopy({
      emailType: 'author-added-to-manuscript',
      titleText,
    }),
  })

  email.sendEmail({ html, text })
}

const sendNewAuthorEmail = ({ email, baseUrl, user, titleText }) => {
  email.toUser = {
    email: user.email,
    name: `${user.firstName} ${user.lastName}`,
  }

  email.content.subject = `Confirm Your Account`
  email.content.unsubscribeLink = services.createUrl(baseUrl, unsubscribeSlug, {
    id: user.id,
  })
  email.content.ctaLink = services.createUrl(baseUrl, resetPath, {
    email: user.email,
    token: user.passwordResetToken,
    firstName: user.firstName,
    lastName: user.lastName,
    affiliation: user.affiliation,
    title: user.title,
  })
  email.content.ctaText = 'CONFIRM ACCOUNT'

  const { html, text } = email.getBody({
    body: getEmailCopy({
      emailType: 'new-author-added-to-manuscript',
      titleText,
    }),
  })

  email.sendEmail({ html, text })
}
